"""
Build article objects by parsing article XML
"""

from __future__ import print_function
from collections import OrderedDict

from elifetools import parseJATS as parser
from elifetools import utils as eautils
from elifearticle import article as ea
from elifearticle import utils


def build_contributors(authors, contrib_type, competing_interests=None):
    """
    Given a list of authors from the parser, instantiate contributors
    objects and build them
    """

    contributors = []

    for author in authors:
        contributor = None
        author_contrib_type = contrib_type

        surname = author.get("surname")
        given_name = author.get("given-names")
        collab = author.get("collab")

        # Small hack for on-behalf-of type when building authors
        #  use on-behalf-of as the contrib_type
        if author.get("type") and author.get("type") == "on-behalf-of":
            collab = author.get("on-behalf-of")
            author_contrib_type = "on-behalf-of"

        if surname or collab or author.get("anonymous"):
            contributor = ea.Contributor(
                author_contrib_type, surname, given_name, collab
            )
            contributor.anonymous = author.get("anonymous")
        else:
            # if no surname then do not add the contributor
            continue

        utils.set_attr_if_value(contributor, "suffix", author.get("suffix"))

        contributor.group_author_key = author.get("group-author-key")
        contributor.orcid = author.get("orcid")
        contributor.corresp = bool(author.get("corresp"))
        if author.get("equal-contrib") == "yes":
            contributor.equal_contrib = True

        # Add contributor affiliations
        for aff in author.get("affiliations", []):
            affiliation = ea.Affiliation()
            # set individual attributes
            utils.set_attr_if_value(affiliation, "city", aff.get("city"))
            utils.set_attr_if_value(affiliation, "country", aff.get("country"))
            utils.set_attr_if_value(affiliation, "department", aff.get("dept"))
            utils.set_attr_if_value(affiliation, "institution", aff.get("institution"))
            utils.set_attr_if_value(affiliation, "ror", aff.get("ror"))
            # generate text value
            affiliation.text = utils.text_from_affiliation_elements(
                aff.get("dept"),
                aff.get("institution"),
                aff.get("city"),
                aff.get("country"),
            )
            # fall back if no other fields are set take the text content
            if affiliation.text == "":
                affiliation.text = aff.get("text")
            contributor.set_affiliation(affiliation)

        # competing interests / conflicts
        if (
            competing_interests
            and author.get("references")
            and "competing-interest" in author.get("references")
        ):
            for ref_id in author["references"]["competing-interest"]:
                for competing_interest in competing_interests:
                    if (
                        competing_interest.get("text")
                        and competing_interest.get("id") == ref_id
                    ):
                        clean_text = utils.remove_tag(
                            "p", competing_interest.get("text")
                        )
                        contributor.set_conflict(clean_text)

        # Finally add the contributor to the list
        if contributor:
            contributors.append(contributor)

    return contributors


def build_funding(award_groups):
    """
    Given a funding data, format it
    """
    if not award_groups:
        return []

    funding_awards = []

    for award_groups_item in award_groups:
        for award_group_id, award_group in award_groups_item.items():
            award = ea.FundingAward()
            award.award_group_id = award_group_id
            if award_group.get("id-type") == "FundRef":
                award.institution_id = award_group.get("id")

            award.institution_name = award_group.get("institution")

            # TODO !!! Check for multiple award_id, if exists
            if award_group.get("award-id"):
                # new Award logic
                award_object = ea.Award()
                award_object.award_id = award_group.get("award-id")
                award_object.award_id_type = award_group.get("award-id-type")
                award.add_award(award_object)

            funding_awards.append(award)

    return funding_awards


def build_datasets(datasets_json):
    """
    Given datasets in JSON format, build and return a list of dataset objects
    """
    if not datasets_json:
        return []

    datasets = []
    dataset_type_map = OrderedDict(
        [("generated", "datasets"), ("used", "prev_published_datasets")]
    )
    dataset_type_map_found = []
    # First look for the types of datasets present
    for dataset_key, dataset_type in dataset_type_map.items():
        if datasets_json.get(dataset_key):
            dataset_type_map_found.append(dataset_key)
    # Continue with the found dataset types
    for dataset_key in dataset_type_map_found:
        dataset_type = dataset_type_map.get(dataset_key)
        for dataset_values in datasets_json.get(dataset_key):
            dataset = ea.Dataset()
            utils.set_attr_if_value(dataset, "dataset_type", dataset_type)
            utils.set_attr_if_value(dataset, "year", dataset_values.get("date"))
            utils.set_attr_if_value(dataset, "title", dataset_values.get("title"))
            utils.set_attr_if_value(dataset, "comment", dataset_values.get("details"))
            utils.set_attr_if_value(dataset, "doi", dataset_values.get("doi"))
            utils.set_attr_if_value(dataset, "uri", dataset_values.get("uri"))
            utils.set_attr_if_value(
                dataset, "accession_id", dataset_values.get("dataId")
            )
            utils.set_attr_if_value(
                dataset, "assigning_authority", dataset_values.get("assigningAuthority")
            )
            # authors
            if dataset_values.get("authors"):
                # parse JSON format authors into author objects
                for author_json in dataset_values.get("authors"):
                    if utils.author_name_from_json(author_json):
                        dataset.add_author(utils.author_name_from_json(author_json))
            # Try to populate the doi attribute if the uri is a doi
            if not dataset.doi and dataset.uri:
                if dataset.uri != eautils.doi_uri_to_doi(dataset.uri):
                    dataset.doi = eautils.doi_uri_to_doi(dataset.uri)
            datasets.append(dataset)
    return datasets


def build_data_availability(datasets_json):
    """
    Given datasets in JSON format, get the data availability from it if present
    """
    data_availability = None
    if "availability" in datasets_json and datasets_json.get("availability"):
        # only expect one paragraph of text
        data_availability = datasets_json.get("availability")[0].get("text")
    return data_availability


def build_ref_list(refs):
    """
    Given parsed references build a list of ref objects
    """
    ref_list = []
    for reference in refs:
        ref = ea.Citation()
        # Publcation Type
        utils.set_attr_if_value(
            ref, "publication_type", reference.get("publication-type")
        )
        # id
        utils.set_attr_if_value(ref, "id", reference.get("id"))
        # Article title
        utils.set_attr_if_value(
            ref, "article_title", reference.get("full_article_title")
        )
        # Source
        utils.set_attr_if_value(ref, "source", reference.get("source"))
        # Volume
        utils.set_attr_if_value(ref, "volume", reference.get("volume"))
        # Issue
        utils.set_attr_if_value(ref, "issue", reference.get("issue"))
        # First page
        utils.set_attr_if_value(ref, "fpage", reference.get("fpage"))
        # Last page
        utils.set_attr_if_value(ref, "lpage", reference.get("lpage"))
        # DOI
        utils.set_attr_if_value(ref, "doi", reference.get("doi"))
        # Year
        utils.set_attr_if_value(ref, "year", reference.get("year"))
        # Year date in iso 8601 format
        utils.set_attr_if_value(
            ref, "year_iso_8601_date", reference.get("year-iso-8601-date")
        )
        # Can set the year_numeric now
        if ref.year_iso_8601_date is not None:
            # First preference take it from the iso 8601 date, if available
            try:
                ref.year_numeric = int(ref.year_iso_8601_date.split("-")[0])
            except ValueError:
                ref.year_numeric = None
        if ref.year_numeric is None:
            # Second preference, use the year value if it is entirely numeric
            if utils.is_year_numeric(ref.year):
                ref.year_numeric = ref.year
        # date-in-citation
        utils.set_attr_if_value(
            ref, "date_in_citation", reference.get("date-in-citation")
        )
        # elocation-id
        utils.set_attr_if_value(ref, "elocation_id", reference.get("elocation-id"))
        # uri
        utils.set_attr_if_value(ref, "uri", reference.get("uri"))
        if not ref.uri:
            # take uri value from uri_text
            utils.set_attr_if_value(ref, "uri", reference.get("uri_text"))
        # pmid
        utils.set_attr_if_value(ref, "pmid", reference.get("pmid"))
        # isbn
        utils.set_attr_if_value(ref, "isbn", reference.get("isbn"))
        # accession
        utils.set_attr_if_value(ref, "accession", reference.get("accession"))
        # patent
        utils.set_attr_if_value(ref, "patent", reference.get("patent"))
        # patent country
        utils.set_attr_if_value(ref, "country", reference.get("country"))
        # publisher-loc
        utils.set_attr_if_value(ref, "publisher_loc", reference.get("publisher_loc"))
        # publisher-name
        utils.set_attr_if_value(ref, "publisher_name", reference.get("publisher_name"))
        # edition
        utils.set_attr_if_value(ref, "edition", reference.get("edition"))
        # version
        utils.set_attr_if_value(ref, "version", reference.get("version"))
        # chapter-title
        utils.set_attr_if_value(ref, "chapter_title", reference.get("chapter-title"))
        # comment
        utils.set_attr_if_value(ref, "comment", reference.get("comment"))
        # data-title
        utils.set_attr_if_value(ref, "data_title", reference.get("data-title"))
        # conf-name
        utils.set_attr_if_value(ref, "conf_name", reference.get("conf-name"))
        # Authors
        if reference.get("authors"):
            for author in reference.get("authors"):
                ref_author = {}
                eautils.set_if_value(ref_author, "group-type", author.get("group-type"))
                eautils.set_if_value(ref_author, "surname", author.get("surname"))
                eautils.set_if_value(
                    ref_author, "given-names", author.get("given-names")
                )
                eautils.set_if_value(ref_author, "collab", author.get("collab"))
                if ref_author:
                    ref.add_author(ref_author)
        # Try to populate the doi attribute if the uri is a doi
        if not ref.doi and ref.uri:
            if ref.uri != eautils.doi_uri_to_doi(ref.uri):
                ref.doi = eautils.doi_uri_to_doi(ref.uri)
        # Append the reference to the list
        ref_list.append(ref)
    return ref_list


def component_title(component):
    """
    Label, title and caption
    Title is the label text plus the title text
    Title may contain italic tag, etc.
    """

    title = ""

    label_text = ""
    title_text = ""
    if component.get("label"):
        label_text = component.get("label")

    if component.get("title"):
        title_text = component.get("title")

    title = str(label_text)
    if label_text != "" and title_text != "":
        title += " "
    title += str(title_text)

    if component.get("type") == "abstract" and title == "":
        title = "Abstract"

    return title


def build_components(components):
    """
    Given parsed components build a list of component objects
    """
    component_list = []

    for comp in components:
        component = ea.Component()

        # id
        component.id = comp.get("id")

        # type
        component.type = comp.get("type")

        # asset, if available
        component.asset = comp.get("asset")

        # DOI
        component.doi = comp.get("doi")

        if component_title(comp) != "":
            component.title = component_title(comp)

        # Subtitle
        if comp.get("type") in ["supplementary-material", "fig"]:

            if comp.get("full_caption"):
                subtitle = comp.get("full_caption")
                subtitle = clean_abstract(subtitle)
                component.subtitle = subtitle

        # Mime type
        if comp.get("type") in [
            "abstract",
            "table-wrap",
            "sub-article",
            "chem-struct-wrap",
            "boxed-text",
        ]:
            component.mime_type = "text/plain"
        if comp.get("type") in ["fig"]:
            component.mime_type = "image/tiff"
        elif comp.get("type") in ["media", "supplementary-material"]:
            if comp.get("mimetype") and comp.get("mime-subtype"):
                component.mime_type = (
                    comp.get("mimetype") + "/" + comp.get("mime-subtype")
                )

        # Permissions
        component.permissions = comp.get("permissions")

        # Append it to our list of components
        component_list.append(component)

    return component_list


def build_review_articles(sub_articles):
    """populate an article review_articles property with data from sub_articles"""
    article_list = []
    for sub_article in sub_articles:
        article = ea.Article(sub_article.get("doi"), sub_article.get("article_title"))
        utils.set_attr_if_value(
            article, "article_type", sub_article.get("article_type")
        )
        utils.set_attr_if_value(article, "id", sub_article.get("id"))
        # contributors
        if sub_article.get("contributors"):
            for contributor in sub_article.get("contributors"):
                article.contributors += build_contributors(
                    [contributor], contributor.get("type")
                )
        # related objects
        if sub_article.get("related_objects"):
            for related_object in sub_article.get("related_objects"):
                related_object_object = ea.RelatedObject()
                utils.set_attr_if_value(
                    related_object_object, "id", related_object.get("id")
                )
                utils.set_attr_if_value(
                    related_object_object, "link_type", related_object.get("link_type")
                )
                utils.set_attr_if_value(
                    related_object_object,
                    "xlink_href",
                    related_object.get("xlink_href"),
                )
                article.related_objects.append(related_object_object)

        # use the parent article license as the sub article license
        license_object = None
        if sub_article.get("parent_license_url"):
            license_object = ea.License()
            license_object.href = sub_article.get("parent_license_url")
            article.license = license_object
        # make the parent article a related article, avoids circular references
        related_article = ea.Article(
            sub_article.get("parent_doi"), sub_article.get("parent_article_title")
        )
        utils.set_attr_if_value(
            related_article, "article_type", sub_article.get("parent_article_type")
        )
        if license_object:
            related_article.license = license_object
        article.related_articles.append(related_article)
        # append article to the list of review articles
        article_list.append(article)
    return article_list


def build_related_articles(related_articles):
    """
    Given parsed data build a list of related article objects
    """
    article_list = []

    for related_article in related_articles:
        article = ea.RelatedArticle()
        if related_article.get("xlink_href"):
            article.xlink_href = related_article.get("xlink_href")
        if related_article.get("related_article_type"):
            article.related_article_type = related_article.get("related_article_type")
        if related_article.get("ext_link_type"):
            article.ext_link_type = related_article.get("ext_link_type")

        # Append it to our list
        article_list.append(article)

    return article_list


def build_pub_dates(article, pub_dates):
    "convert pub_dates into ArticleDate objects and add them to article"
    for pub_date in pub_dates:
        # always want a date type, take it from pub-type if must
        if pub_date.get("date-type"):
            date_instance = ea.ArticleDate(
                pub_date.get("date-type"), pub_date.get("date")
            )
        elif pub_date.get("pub-type"):
            date_instance = ea.ArticleDate(
                pub_date.get("pub-type"), pub_date.get("date")
            )
        # Set more values
        utils.set_attr_if_value(date_instance, "pub_type", pub_date.get("pub-type"))
        utils.set_attr_if_value(
            date_instance, "publication_format", pub_date.get("publication-format")
        )
        utils.set_attr_if_value(date_instance, "day", pub_date.get("day"))
        utils.set_attr_if_value(date_instance, "month", pub_date.get("month"))
        utils.set_attr_if_value(date_instance, "year", pub_date.get("year"))
        article.add_date(date_instance)


def build_self_uri_list(self_uri_list):
    "parse the self-uri tags, build Uri objects"
    uri_list = []
    for self_uri in self_uri_list:
        uri = ea.Uri()
        utils.set_attr_if_value(uri, "xlink_href", self_uri.get("xlink_href"))
        utils.set_attr_if_value(uri, "content_type", self_uri.get("content-type"))
        uri_list.append(uri)
    return uri_list


def build_preprint(pub_history_events):
    preprint = ea.Preprint()
    for event in pub_history_events:
        if event.get("event_type") == "preprint":
            preprint.uri = event.get("uri")
            # Try to populate the doi attribute if the uri is a doi
            if preprint.uri and preprint.uri != eautils.doi_uri_to_doi(preprint.uri):
                preprint.doi = eautils.doi_uri_to_doi(preprint.uri)
            break
    if preprint.uri or preprint.doi:
        return preprint
    return None


def build_publication_history(pub_history_events):
    event_list = []
    for event in pub_history_events:
        event_object = ea.Event()
        event_object.uri = event.get("uri")
        # Try to populate the doi attribute if the uri is a doi
        if event_object.uri and event_object.uri != eautils.doi_uri_to_doi(
            event_object.uri
        ):
            event_object.doi = eautils.doi_uri_to_doi(event_object.uri)
        # more Event properties
        event_object.event_type = event.get("event_type")
        event_object.event_desc = event.get("event_desc")
        event_object.date = event.get("date")
        # add to the list of events
        event_list.append(event_object)
    return event_list


def build_clinical_trials(clinical_trials):
    clinical_trial_list = []
    for clinical_trial in clinical_trials:
        clinical_trial_object = ea.ClinicalTrial()
        attr_map = {
            "id": "id",
            "content-type": "content_type",
            "document-id": "document_id",
            "document-id-type": "document_id_type",
            "source-id": "source_id",
            "source-id-type": "source_id_type",
            "source-type": "source_type",
            "text": "text",
            "xlink_href": "xlink_href",
        }
        for dict_key, attr_name in attr_map.items():
            utils.set_attr_if_value(
                clinical_trial_object, attr_name, clinical_trial.get(dict_key)
            )
        clinical_trial_list.append(clinical_trial_object)
    return clinical_trial_list


def clean_abstract(abstract, remove_tags=None):
    """
    Remove unwanted tags from abstract string,
    parsing it as HTML, then only keep the body paragraph contents
    """
    if remove_tags is None:
        remove_tags = ["xref", "ext-link", "inline-formula", "mml:*"]
    if remove_tags:
        for tag_name in remove_tags:
            abstract = utils.remove_tag(tag_name, abstract)
    return abstract


def build_part_check(part, build_parts):
    """
    check if only specific parts were specified to be build when parsing
    if the list build_parts is empty, then all parts will be parsed
    """
    if not build_parts:
        return True
    return bool(part in build_parts)


def build_article_from_xml(
    article_xml_filename, detail="brief", build_parts=None, remove_tags=None
):
    """
    Parse JATS XML with elifetools parser, and populate an
    eLifePOA article object
    Basic data crossref needs: article_id, doi, title, contributors with names set
    detail="brief" is normally enough,
    detail="full" will populate all the contributor affiliations that are linked by xref tags
    """

    def build_part(part):
        return build_part_check(part, build_parts)

    error_count = 0

    soup = parser.parse_document(article_xml_filename)

    # Get DOI
    doi = parser.doi(soup)

    # Create the article object
    article = ea.Article(doi, title=None)

    # article version from the filename if possible
    utils.set_attr_if_value(
        article, "version", utils.version_from_xml_filename(article_xml_filename)
    )

    # version doi
    if build_part("basic"):
        article.version_doi = parser.version_doi(soup)

    # journal title
    if build_part("basic"):
        article.journal_title = parser.journal_title(soup)

    # issn
    if build_part("basic"):
        article.journal_issn = parser.journal_issn(soup, "electronic")
        if article.journal_issn is None:
            article.journal_issn = parser.journal_issn(soup)

    # Related articles
    if build_part("related_articles"):
        article.related_articles = build_related_articles(parser.related_article(soup))

    # Get publisher_id pii
    if build_part("basic"):
        article.pii = parser.publisher_id(soup)

    # set object manuscript value
    if build_part("basic"):
        manuscript = parser.publisher_id(soup)
        if not manuscript and doi:
            # try to get it from the DOI
            manuscript = doi.split(".")[-1]
        article.manuscript = manuscript

    # Set the articleType
    if build_part("basic"):
        article_type = parser.article_type(soup)
        if article_type:
            article.article_type = article_type

    # title
    if build_part("basic"):
        article.title = parser.full_title(soup)
    # print article.title

    # publisher_name
    if build_part("basic"):
        article.publisher_name = parser.publisher(soup)

    # publisher_name
    if build_part("basic"):
        article.clinical_trials = build_clinical_trials(parser.clinical_trials(soup))

    # abstract
    if build_part("abstract"):
        article.abstract = clean_abstract(parser.full_abstract(soup), remove_tags)
        article.abstract_json = parser.abstract_json(soup)
        article.abstract_xml = parser.abstract_xml(soup)

    # digest
    if build_part("abstract"):
        article.digest = clean_abstract(parser.full_digest(soup), remove_tags)

    # elocation-id
    if build_part("basic"):
        article.elocation_id = parser.elocation_id(soup)

    # issue
    if build_part("basic"):
        article.issue = parser.issue(soup)

    # self-uri
    if build_part("basic"):
        article.self_uri_list = build_self_uri_list(parser.self_uri(soup))

    # preprint
    if build_part("basic"):
        article.preprint = build_preprint(parser.pub_history(soup))

    # publication history events
    if build_part("basic"):
        article.publication_history = build_publication_history(
            parser.pub_history(soup)
        )

    # contributors
    if build_part("contributors"):
        # get the competing interests if available
        competing_interests = parser.competing_interests(soup, None)
        all_contributors = parser.contributors(soup, detail)
        author_contributors = [
            con
            for con in all_contributors
            if con.get("type") in ["author", "on-behalf-of"]
        ]
        contrib_type = "author"
        contributors = build_contributors(
            author_contributors, contrib_type, competing_interests
        )

        contrib_type = "author non-byline"
        authors = parser.authors_non_byline(soup, detail)
        contributors_non_byline = build_contributors(
            authors, contrib_type, competing_interests
        )
        article.contributors = contributors + contributors_non_byline

        # also populate the editors when contributors flag is being built
        editor_contrib_types = ["assoc_ed", "editor", "reviewer", "senior_editor"]
        for contrib_type in editor_contrib_types:
            editor_contributors = [
                con for con in all_contributors if con.get("type") == contrib_type
            ]
            article.editors += build_contributors(editor_contributors, contrib_type)

    # license href
    if build_part("license"):
        license_object = ea.License()
        license_object.href = parser.license_url(soup)
        license_object.copyright_statement = parser.copyright_statement(soup)
        article.license = license_object

    # article_category
    if build_part("categories"):
        article.article_categories = parser.category(soup)

    # display channel
    if build_part("categories"):
        article.display_channel = eautils.firstnn(parser.display_channel(soup))

    # keywords
    if build_part("keywords"):
        article.author_keywords = parser.keywords(soup)

    # research organisms
    if build_part("research_organisms"):
        article.research_organisms = parser.research_organism(soup)

    # funding awards
    if build_part("funding"):
        article.funding_awards = build_funding(parser.full_award_groups(soup))

    # datasets
    if build_part("datasets"):
        datasets_json = parser.datasets_json(soup)
        article.datasets = build_datasets(datasets_json)
        article.data_availability = build_data_availability(datasets_json)

    # references or citations
    if build_part("references"):
        article.ref_list = build_ref_list(parser.refs(soup))

    # components with component DOI
    if build_part("components"):
        article.component_list = build_components(parser.components(soup))

    # History dates
    if build_part("history"):
        date_types = ["received", "accepted"]
        for date_type in date_types:
            history_date = parser.history_date(soup, date_type)
            if history_date:
                date_instance = ea.ArticleDate(date_type, history_date)
                article.add_date(date_instance)

    # Pub date
    if build_part("pub_dates"):
        build_pub_dates(article, parser.pub_dates(soup))

    # Set the volume if present
    if build_part("volume"):
        volume = parser.volume(soup)
        if volume:
            article.volume = volume

    if build_part("is_poa"):
        article.is_poa = parser.is_poa(soup)

    # peer review articles
    if build_part("sub_articles"):
        article.review_articles = build_review_articles(parser.sub_articles(soup))

    return article, error_count


def build_articles_from_article_xmls(
    article_xmls, detail="full", build_parts=None, remove_tags=None
):
    """
    Given a list of article XML filenames, convert to article objects
    """

    poa_articles = []

    for article_xml in article_xmls:
        print("working on ", article_xml)
        article, error_count = build_article_from_xml(
            article_xml, detail, build_parts, remove_tags
        )
        if error_count == 0:
            poa_articles.append(article)

    return poa_articles



import unittest
import warnings
import time
from elifearticle import article as ea


def generate_date(date_string="2013-10-03", date_format="%Y-%m-%d"):
    return time.strptime(date_string, date_format)


class TestArticle(unittest.TestCase):
    def setUp(self):
        self.article = ea.Article()

    def add_dates(self):
        "for reuse in testing add_date and get_date, add these dates"
        date_accepted = ea.ArticleDate("accepted", generate_date("2017-07-02"))
        date_received = ea.ArticleDate("received", generate_date("2017-07-04"))
        self.article.add_date(date_accepted)
        self.article.add_date(date_received)

    def add_datasets(self):
        "for reuse in testing add_dataset and get_datasets, add these datasets"
        for dataset_type in ["datasets", "prev_published_datasets"]:
            dataset = ea.Dataset()
            dataset.dataset_type = dataset_type
            self.article.add_dataset(dataset)

    def test_article_init(self):
        self.assertEqual(self.article.article_type, "research-article")

    def test_add_contributor(self):
        contributor = None
        self.article.add_contributor(contributor)
        self.assertEqual(len(self.article.contributors), 1)

    def test_add_research_organism(self):
        research_organism = None
        self.article.add_research_organism(research_organism)
        self.assertEqual(len(self.article.research_organisms), 1)

    def test_add_date(self):
        self.add_dates()
        self.assertEqual(len(self.article.dates), 2)

    def test_get_date(self):
        self.add_dates()
        date_accepted = self.article.get_date("accepted")
        date_received = self.article.get_date("received")
        not_a_date = self.article.get_date("not_a_date")
        self.assertEqual(date_accepted.date, generate_date("2017-07-02"))
        self.assertEqual(date_received.date, generate_date("2017-07-04"))
        self.assertEqual(not_a_date, None)

    def test_get_display_channel(self):
        self.assertEqual(self.article.get_display_channel(), None)

    def test_add_article_category(self):
        category = None
        self.article.add_article_category(category)
        self.assertEqual(len(self.article.article_categories), 1)

    def test_has_contributor_conflict(self):
        self.assertEqual(self.article.has_contributor_conflict(), False)
        author = ea.Contributor("author", "Insect", "Amber")
        author.set_conflict("A conflict")
        self.article.add_contributor(author)
        self.assertEqual(self.article.has_contributor_conflict(), True)

    def test_add_ethic(self):
        ethic = None
        self.article.add_ethic(ethic)
        self.assertEqual(len(self.article.ethics), 1)

    def test_add_author_keyword(self):
        keyword = None
        self.article.add_author_keyword(keyword)
        self.assertEqual(len(self.article.author_keywords), 1)

    def test_add_dataset(self):
        self.add_datasets()
        self.assertEqual(len(self.article.datasets), 2)

    def test_get_datasets(self):
        self.add_datasets()
        self.assertEqual(len(self.article.get_datasets()), 2)
        self.assertEqual(len(self.article.get_datasets("datasets")), 1)
        self.assertEqual(len(self.article.get_datasets("prev_published_datasets")), 1)

    def test_add_funding_award(self):
        funding_award = None
        self.article.add_funding_award(funding_award)
        self.assertEqual(len(self.article.funding_awards), 1)

    def test_unicode(self):
        "for test coverage of lists and dict"
        self.article.datasets = []
        self.article.license = {}
        self.assertIsNotNone(str(self.article))

    def test_add_self_uri(self):
        uri = ea.Uri()
        self.article.add_self_uri(uri)
        self.assertEqual(len(self.article.self_uri_list), 1)

    def test_get_self_uri(self):
        uri = ea.Uri()
        self.article.add_self_uri(uri)
        uri2 = ea.Uri()
        uri2.content_type = "pdf"
        self.article.add_self_uri(uri2)
        self.assertIsNotNone(self.article.get_self_uri(None))
        self.assertIsNotNone(self.article.get_self_uri("pdf"))
        self.assertIsNone(self.article.get_self_uri("not_a_valid_type"))


class TestArticleDate(unittest.TestCase):
    def test_article_date_init(self):
        article_date = ea.ArticleDate("test", generate_date())
        self.assertEqual(article_date.date_type, "test")


class TestContributor(unittest.TestCase):
    def setUp(self):
        self.contributor = ea.Contributor("author", "Insect", "Amber")

    def test_contributor_init(self):
        self.assertEqual(self.contributor.contrib_type, "author")

    def test_set_affiliation(self):
        affiliation = None
        self.contributor.set_affiliation(affiliation)
        self.assertEqual(len(self.contributor.affiliations), 1)

    def test_set_conflict(self):
        conflict = "A conflict"
        self.contributor.set_conflict(conflict)
        self.assertEqual(self.contributor.conflict, [conflict])
        # add another one
        self.contributor.set_conflict(conflict)
        self.assertEqual(self.contributor.conflict, [conflict, conflict])


class TestAffiliation(unittest.TestCase):
    def setUp(self):
        self.affiliation = ea.Affiliation()

    def test_affilation_init(self):
        self.assertIsNotNone(self.affiliation)


class TestRole(unittest.TestCase):
    def test_role_init(self):
        text = "Reviewing Editor"
        specific_use = "editor"
        role = ea.Role(text, specific_use)
        expected = {"text": text, "specific_use": specific_use}
        self.assertEqual(str(role), str(expected))


class TestDataset(unittest.TestCase):
    def setUp(self):
        self.dataset = ea.Dataset()

    def test_dataset_init(self):
        self.assertIsNotNone(self.dataset)

    def test_add_author(self):
        author = None
        self.dataset.add_author(author)
        self.assertEqual(len(self.dataset.authors), 1)


class TestAward(unittest.TestCase):
    def setUp(self):
        self.award = ea.Award()

    def test_award_init(self):
        self.assertIsNotNone(self.award)


class TestFundingAward(unittest.TestCase):
    def setUp(self):
        self.funding_award = ea.FundingAward()

    def test_funding_award_init(self):
        self.assertIsNotNone(self.funding_award)

    def test_add_principal_award_recipient(self):
        principal_award_recipient = None
        self.funding_award.add_principal_award_recipient(principal_award_recipient)
        self.assertEqual(len(self.funding_award.principal_award_recipients), 1)

    def test_get_funder_identifier(self):
        institution_id = "http://dx.doi.org/10.13039/100004440"
        self.funding_award.institution_id = institution_id
        self.assertEqual(self.funding_award.get_funder_identifier(), "100004440")

    def test_get_funder_identifier_fail(self):
        # test failure
        institution_id = 1
        self.funding_award.institution_id = institution_id
        self.assertEqual(self.funding_award.get_funder_identifier(), None)

        # test success
        institution_id = "http://dx.doi.org/10.13039/100004440"
        self.funding_award.institution_id = institution_id
        self.assertEqual(self.funding_award.get_funder_identifier(), "100004440")

    def test_get_funder_name(self):
        institution_name = "institution_name"
        self.funding_award.institution_name = institution_name
        self.assertEqual(self.funding_award.get_funder_name(), institution_name)


class TestLicense(unittest.TestCase):
    def setUp(self):
        self.license = ea.License()

    def test_license_init(self):
        self.assertIsNotNone(self.license)

    def test_license_1(self):
        license_object = ea.License(1)
        self.assertIsNotNone(license_object)

    def test_license_2(self):
        license_object = ea.License(2)
        self.assertIsNotNone(license_object)


class TestCitation(unittest.TestCase):
    def setUp(self):
        self.citation = ea.Citation()

    def test_citation_init(self):
        self.assertIsNotNone(self.citation)

    def test_add_author(self):
        author = None
        self.citation.add_author(author)
        self.assertEqual(len(self.citation.authors), 1)

    def test_get_journal_title(self):
        source = "source_name"
        self.citation.source = source
        self.assertEqual(self.citation.get_journal_title(), source)


class TestComponent(unittest.TestCase):
    def setUp(self):
        self.component = ea.Component()

    def test_component_init(self):
        self.assertIsNotNone(self.component)


class TestRelatedArticle(unittest.TestCase):
    def setUp(self):
        self.related_article = ea.RelatedArticle()

    def test_related_article_init(self):
        self.assertIsNotNone(self.related_article)


class TestContentBlock(unittest.TestCase):
    def test_content_block(self):
        """test blank content block"""
        content_block = ea.ContentBlock()
        self.assertEqual(content_block.attr_names(), [])

    def test_content_block_attr(self):
        """test attributes function"""
        content_block = ea.ContentBlock(
            "disp-quote", None, {"content-type": "editor-comment"}
        )
        content_block.attr["escaped"] = '"'
        self.assertEqual(
            sorted(content_block.attr_names()), ["content-type", "escaped"]
        )
        self.assertEqual(
            content_block.attr_string(),
            ' content-type="editor-comment" escaped="&quot;"',
        )


if __name__ == "__main__":
    unittest.main()



import unittest
import re
import os
import time
from ddt import ddt, data, unpack
from elifearticle import utils


@ddt
class TestUtils(unittest.TestCase):
    def setUp(self):
        pass

    def test_repl(self):
        string = "&#x2022;"
        matches = re.match(r"&#x(....);", string)
        self.assertEqual(utils.repl(matches), "\u2022")

    def test_entity_to_unicode(self):
        passes = []
        passes.append(
            (
                "N-terminal &#x03B1;-helix into the heterodimer interface",
                "N-terminal \u03b1-helix into the heterodimer interface",
            )
        )

        passes.append(
            (
                "N-terminal &alpha;-helix into the heterodimer interface",
                "N-terminal \u03b1-helix into the heterodimer interface",
            )
        )

        passes.append(
            (
                (
                    "&#x00A0; &#x00C5; &#x00D7; &#x00EF; &#x0394; &#x03B1; &#x03B2; &#x03B3;"
                    " &#x03BA; &#x03BB; &#x2212; &#x223C; &alpha; &amp; &beta; &epsilon; &iuml;"
                    " &ldquo; &ordm; &rdquo;"
                ),
                (
                    "\xa0 \xc5 \xd7 \xef \u0394 \u03b1 \u03b2 \u03b3"
                    " \u03ba \u03bb \u2212 \u223c \u03b1 &amp; \u03b2 \u03b5 \xcf "
                    '" \xba "'
                ),
            )
        )

        for string_input, string_output in passes:
            self.assertEqual(utils.entity_to_unicode(string_input), string_output)

    def test_remove_tag(self):
        self.assertEqual(utils.remove_tag("i", "<i>test</i>"), "test")
        self.assertEqual(utils.remove_tag("i", None), None)

    def test_replace_tags(self):
        self.assertEqual(utils.replace_tags("<i>"), "<italic>")

    def test_version_from_xml_filename(self):
        self.assertEqual(utils.version_from_xml_filename(None), None)
        self.assertEqual(utils.version_from_xml_filename("elife-00666.xml"), None)
        self.assertEqual(utils.version_from_xml_filename("elife-02935-v2.xml"), 2)
        self.assertEqual(
            utils.version_from_xml_filename(
                os.path.join("test-folder", "elife-02935-v2.xml")
            ),
            2,
        )
        self.assertEqual(utils.version_from_xml_filename("bmjopen-4-e003269.xml"), None)
        self.assertEqual(
            utils.version_from_xml_filename("elife-preprint-84364-v2.xml"), 2
        )

    def test_calculate_journal_volume(self):
        "for test coverage"
        self.assertEqual(utils.calculate_journal_volume(None, None), None)
        pub_date = time.strptime("2017-01-01", "%Y-%m-%d")
        self.assertEqual(utils.calculate_journal_volume(pub_date, 2017), "1")
        self.assertEqual(utils.calculate_journal_volume(pub_date, None), None)

    def test_get_last_commit_to_master(self):
        self.assertIsNotNone(utils.get_last_commit_to_master())

    def test_get_last_commit_to_master_no_path(self):
        git_path = "not_a_path"
        last_commit = ""
        expected = "None"
        last_commit = utils.get_last_commit_to_master(git_path)
        self.assertEqual(last_commit, expected)

    @unpack
    @data(
        (None, None, None, None, ""),
        ("", None, None, None, ""),
        ("One", "Two", "Three", "Four", "One, Two, Three, Four"),
        ("One", "Two", None, "Four", "One, Two, Four"),
        ("One", "Two", "", "Four", "One, Two, Four"),
    )
    def test_text_from_affiliation_elements(
        self, department, institution, city, country, expected
    ):
        self.assertEqual(
            utils.text_from_affiliation_elements(
                department, institution, city, country
            ),
            expected,
            "{expected} not found testing {department}, {institution}, {city}, {country}".format(
                expected=expected,
                department=department,
                institution=institution,
                city=city,
                country=country,
            ),
        )


class TestUtilsAttr(unittest.TestCase):
    def setUp(self):
        self.attr_map = {"foo": "& bar", "more": '"complicated"'}

    def test_attr_names(self):
        self.assertEqual(utils.attr_names(self.attr_map), ["foo", "more"])

    def test_attr_names_blank(self):
        self.assertEqual(utils.attr_names(None), [])

    def test_attr_string(self):
        expected = ' foo="&amp; bar" more="&quot;complicated&quot;"'
        self.assertEqual(utils.attr_string(self.attr_map), expected)

    def test_attr_blank(self):
        self.assertEqual(utils.attr_string(None), "")


if __name__ == "__main__":
    unittest.main()



import unittest
from collections import OrderedDict
from elifearticle import parse


class TestParseBuildDatasets(unittest.TestCase):
    def setUp(self):
        pass

    def test_datasets_uri_to_doi(self):
        "test converting uri to doi value"
        # based on a dataset in elife-01201-v1
        datasets_data = {
            "generated": [
                {"uri": "http://www.ncbi.nlm.nih.gov/geo/query/acc.cgi?acc=GSE51740"},
                {"uri": "http://dx.doi.org/10.5061/dryad.cv39v"},
            ]
        }
        datasets = parse.build_datasets(datasets_data)
        self.assertEqual(datasets[0].doi, None)
        self.assertEqual(datasets[1].doi, "10.5061/dryad.cv39v")

    def test_build_data_availability(self):
        "test extracting the data availability statement"
        statement = "Availability statement"
        expected = statement
        datasets_data = OrderedDict(
            [
                (
                    "availability",
                    [OrderedDict([("type", "paragraph"), ("text", statement)])],
                )
            ]
        )
        data_availability = parse.build_data_availability(datasets_data)
        self.assertEqual(data_availability, expected)

    def test_build_data_availability_no_value(self):
        "test extracting data availability encountering an IndexError"
        expected = None
        datasets_data = OrderedDict([("availability", "")])
        data_availability = parse.build_data_availability(datasets_data)
        self.assertEqual(data_availability, expected)



import unittest
import os

from elifearticle import parse
from tests import XLS_PATH


class TestParseBuildRefList(unittest.TestCase):
    def setUp(self):
        pass

    def test_build_ref_list_666(self):
        xml_files = [os.path.join(XLS_PATH, "elife-00666.xml")]
        articles = parse.build_articles_from_article_xmls(xml_files)
        article = articles[0]

        # Now check various details as desired
        # ref bib1 details
        self.assertEqual(article.ref_list[0].id, "bib1")
        self.assertEqual(
            article.ref_list[0].article_title,
            "TIP47 is a key effector for Rab9 localization",
        )
        self.assertEqual(article.ref_list[0].source, "The Journal of Cell Biology")
        self.assertEqual(article.ref_list[0].volume, "173")
        # note: no test for issue attribute yet
        self.assertEqual(article.ref_list[0].fpage, "917")
        self.assertEqual(article.ref_list[0].lpage, "926")
        self.assertEqual(article.ref_list[0].doi, "10.1083/jcb.200510010")
        self.assertEqual(article.ref_list[0].year, "2006")
        self.assertEqual(article.ref_list[0].pmid, "16769818")
        # ref bib2 details
        self.assertEqual(
            article.ref_list[1].uri,
            "https://cran.r-project.org/web/packages/lme4/index.html",
        )
        self.assertEqual(article.ref_list[1].version, "1.1-12")
        self.assertEqual(
            article.ref_list[1].data_title,
            "Lme4: Linear Mixed-Effects Models Using Eigen and S4",
        )
        # ref bib4 details
        self.assertEqual(article.ref_list[3].accession, "GSE44902")
        # ref bib7 details
        self.assertEqual(article.ref_list[6].publisher_loc, "Cambridge, UK")
        self.assertEqual(article.ref_list[6].publisher_name, "Global Phasing Ltd")
        # ref bib10 details
        self.assertEqual(article.ref_list[9].edition, "2")
        # ref bib11 details
        self.assertEqual(article.ref_list[10].publication_type, "book")
        self.assertEqual(article.ref_list[10].chapter_title, "Two rules of speciation")
        # ref bib11 all authors
        self.assertEqual(len(article.ref_list[10].authors), 4)
        # ref bib11 authors of type author
        self.assertEqual(
            len(
                [
                    c
                    for c in article.ref_list[10].authors
                    if c.get("group-type") == "author"
                ]
            ),
            2,
        )
        # ref bib11 authors of type editor
        self.assertEqual(
            len(
                [
                    c
                    for c in article.ref_list[10].authors
                    if c.get("group-type") == "editor"
                ]
            ),
            2,
        )
        # ref bib13 details
        self.assertEqual(article.ref_list[12].isbn, "9781474130158")
        # ref bib15 details
        self.assertEqual(article.ref_list[14].publication_type, "web")
        self.assertEqual(article.ref_list[14].date_in_citation, "August 19, 2016")
        # ref bib24 details
        self.assertEqual(
            article.ref_list[23].conf_name,
            "IEEE Conference on Computer Vision and Pattern Recognition (CVPR), 2010",
        )
        # ref bib32 details
        self.assertEqual(article.ref_list[31].patent, "US20100941530")
        self.assertEqual(article.ref_list[31].country, "United States")
        # ref bib37 details
        self.assertEqual(article.ref_list[36].comment, "In press")
        # ref bib40 details
        self.assertEqual(article.ref_list[39].year_iso_8601_date, "1993-09-09")
        self.assertEqual(article.ref_list[39].year_numeric, 1993)
        # ref bib46 author is a collab
        self.assertEqual(
            article.ref_list[45].authors[0].get("collab"),
            "The <italic>Shigella</italic> Genome Sequencing Consortium",
        )
        # ref bib51 details
        self.assertEqual(article.ref_list[50].elocation_id, "e149")


class TestBuildRefList(unittest.TestCase):
    def test_uri_to_doi(self):
        "test converting a uri to doi value when applicable"
        # test based on elife-20047-v2.xml bib14
        refs = [{"uri": "http://dx.doi.org/ 10.5061/dryad.r1072"}]
        ref_list = parse.build_ref_list(refs)
        self.assertEqual(ref_list[0].doi, " 10.5061/dryad.r1072")

    def test_bad_year_iso_8601_date(self):
        """
        based on elife 09771 v3 bib22 has a non-numeric year in the following attribute
        <year iso-8601-date="2012, . 2011">2012, . 2011</year>
        this test simulates the result when converted to a Citation object
        """
        refs = [{"year-iso-8601-date": "2012, . 2011"}]
        ref_list = parse.build_ref_list(refs)
        self.assertEqual(ref_list[0].year_numeric, None)



import unittest
from collections import OrderedDict
from elifearticle import parse


REGISTRY_NAME_TO_DOI_MAP = OrderedDict(
    [("ClinicalTrials.gov", "10.18810/clinical-trials-gov")]
)


class TestParseBuildClinicalTrials(unittest.TestCase):
    def setUp(self):
        self.clinical_trials_data = [
            OrderedDict(
                [
                    ("id", "CT1"),
                    ("content-type", "post-result"),
                    ("document-id", "NCT02836002"),
                    ("document-id-type", "clinical-trial-number"),
                    ("source-id", "ClinicalTrials.gov"),
                    ("source-id-type", "registry-name"),
                    ("source-type", "clinical-trials-registry"),
                    ("text", "NCT02836002"),
                    ("xlink_href", "https://clinicaltrials.gov/show/NCT02836002"),
                ]
            ),
            OrderedDict(
                [
                    ("id", "CT1"),
                    ("content-type", "preResult"),
                    ("document-id", "NCT04094727"),
                    ("document-id-type", "clinical-trial-number"),
                    ("source-id", "10.18810/clinical-trials-gov"),
                    ("source-id-type", "crossref-doi"),
                    ("source-type", "clinical-trials-registry"),
                    ("text", "NCT04094727"),
                    ("xlink_href", "https://clinicaltrials.gov/show/NCT04094727"),
                ]
            ),
        ]

    def test_build_clinical_trials(self):
        "test building ClinicalTrial objects from clinical trials data"
        clinical_trials = parse.build_clinical_trials(self.clinical_trials_data)
        self.assertEqual(len(clinical_trials), 2)
        # spot check a few values
        self.assertEqual(
            clinical_trials[0].xlink_href, "https://clinicaltrials.gov/show/NCT02836002"
        )
        self.assertEqual(clinical_trials[0].get_registry_doi(), None)
        self.assertEqual(clinical_trials[1].document_id, "NCT04094727")
        self.assertEqual(
            clinical_trials[1].get_registry_doi(), "10.18810/clinical-trials-gov"
        )

    def test_build_clinical_trials_registry_doi_map(self):
        "build clinicial trials data then get a registry doi using the registry name to doi map"
        clinical_trials = parse.build_clinical_trials(self.clinical_trials_data)
        # override a value for checking test coverage
        registry_doi = "injected_value"
        clinical_trials[1].registry_doi = registry_doi
        self.assertEqual(
            clinical_trials[0].get_registry_doi(REGISTRY_NAME_TO_DOI_MAP),
            "10.18810/clinical-trials-gov",
        )
        self.assertEqual(
            clinical_trials[1].get_registry_doi(REGISTRY_NAME_TO_DOI_MAP), registry_doi
        )



import unittest
import os
from collections import OrderedDict
from elifearticle import parse
from elifearticle.article import Preprint
from tests import XLS_PATH


class TestParseXml(unittest.TestCase):
    def setUp(self):
        self.passes = []
        self.passes.append(os.path.join(XLS_PATH, "elife-02935-v2.xml"))
        self.passes.append(os.path.join(XLS_PATH, "elife-04637-v2.xml"))
        self.passes.append(os.path.join(XLS_PATH, "elife-15743-v1.xml"))
        self.passes.append(os.path.join(XLS_PATH, "elife-02043-v2.xml"))
        self.passes.append(os.path.join(XLS_PATH, "elife-14003.xml"))
        self.passes.append(os.path.join(XLS_PATH, "elife-00666.xml"))
        self.passes.append(os.path.join(XLS_PATH, "cstp77-jats.xml"))

    def test_parse(self):
        articles = parse.build_articles_from_article_xmls(self.passes)
        self.assertEqual(len(articles), 7)

    def test_parse_build_parts_default(self):
        "test parse build parts"

        def check_article(article):
            "function for repeatable article assertions for the two builds"
            self.assertNotEqual(article.abstract, "")
            self.assertNotEqual(article.elocation_id, "")
            self.assertGreater(len(article.article_categories), 0)
            self.assertGreater(len(article.component_list), 0)
            self.assertGreater(len(article.contributors), 0)
            self.assertGreater(len(article.funding_awards), 0)
            self.assertGreater(len(article.dates), 0)
            self.assertIsNotNone(article.get_date("received"))
            self.assertIsNotNone(article.is_poa)
            self.assertGreater(len(article.author_keywords), 0)
            self.assertIsNotNone(article.get_date("pub"))

        def check_article_15743(article):
            "function for repeatable article assertions for the two builds"
            self.assertGreater(len(article.related_articles), 0)

        # first, default build parts should build all parts
        article_xmls = [os.path.join(XLS_PATH, "elife-02043-v2.xml")]
        article = parse.build_articles_from_article_xmls(article_xmls)[0]
        check_article(article)
        # also check building an article with a related article
        article_xmls = [os.path.join(XLS_PATH, "elife-15743-v1.xml")]
        article = parse.build_articles_from_article_xmls(article_xmls)[0]
        check_article_15743(article)
        # second, set all the build parts and the result should be the same
        detail = "full"
        build_parts = [
            "abstract",
            "basic",
            "categories",
            "components",
            "contributors",
            "datasets",
            "funding",
            "history",
            "is_poa",
            "keywords",
            "license",
            "pub_dates",
            "references",
            "related_articles",
            "research_organisms",
            "volume",
            "sub_articles",
        ]
        article_xmls = [os.path.join(XLS_PATH, "elife-02043-v2.xml")]
        article = parse.build_articles_from_article_xmls(
            article_xmls, detail, build_parts
        )[0]
        check_article(article)
        # also check building an article with a related article
        article_xmls = [os.path.join(XLS_PATH, "elife-15743-v1.xml")]
        article = parse.build_articles_from_article_xmls(
            article_xmls, detail, build_parts
        )[0]
        check_article_15743(article)

    def test_parse_build_parts_basic(self):
        "test building with very basic build parts"
        detail = "brief"
        build_parts = ["basic"]
        article_xmls = [os.path.join(XLS_PATH, "elife-02043-v2.xml")]
        article = parse.build_articles_from_article_xmls(
            article_xmls, detail, build_parts
        )[0]
        # check the result
        # elocation_id will exist, but not other parts we check
        self.assertNotEqual(article.elocation_id, "")
        self.assertEqual(article.abstract, "")
        self.assertEqual(len(article.article_categories), 0)
        self.assertIsNone(article.display_channel)
        self.assertEqual(len(article.component_list), 0)
        self.assertEqual(len(article.contributors), 0)
        self.assertEqual(len(article.funding_awards), 0)
        self.assertEqual(article.dates, {})
        self.assertIsNone(article.get_date("received"))
        self.assertIsNone(article.is_poa)
        self.assertEqual(len(article.author_keywords), 0)
        self.assertIsNone(article.get_date("pub"))
        self.assertEqual(len(article.related_articles), 0)
        # also check building an article with a related article
        article_xmls = [os.path.join(XLS_PATH, "elife-15743-v1.xml")]
        article = parse.build_articles_from_article_xmls(
            article_xmls, detail, build_parts
        )[0]
        self.assertEqual(len(article.related_articles), 0)


class TestBuildContributors(unittest.TestCase):
    def test_build_contributors(self):
        "test for when a contributor has no surname"
        authors = [{"given-name": "Foo"}]
        contrib_type = "author"
        self.assertEqual(parse.build_contributors(authors, contrib_type), [])

    def test_build_contributors_affiliations(self):
        "test for contributor with affiliations"
        aff = {
            "dept": "A Test Department",
            "institution": "School of Biological Sciences, University of Bristol",
            "city": "Bristol",
            "country": "United Kingdom",
            "ror": "https://ror.org/0524sp257",
        }
        authors = [
            {
                "surname": "Bar",
                "affiliations": [aff],
            }
        ]
        contrib_type = "author"
        contributors = parse.build_contributors(authors, contrib_type)
        affiliation_object = contributors[0].affiliations[0]
        self.assertEqual(getattr(affiliation_object, "department"), aff.get("dept"))
        self.assertEqual(
            getattr(affiliation_object, "institution"), aff.get("institution")
        )
        self.assertEqual(getattr(affiliation_object, "city"), aff.get("city"))
        self.assertEqual(getattr(affiliation_object, "country"), aff.get("country"))
        self.assertEqual(getattr(affiliation_object, "ror"), aff.get("ror"))
        self.assertEqual(
            getattr(affiliation_object, "text"),
            "%s, %s, %s, %s"
            % (
                aff.get("dept"),
                aff.get("institution"),
                aff.get("city"),
                aff.get("country"),
            ),
        )

    def test_build_anonymous_contributor(self):
        "test for an anonymous contributor"
        authors = [{"anonymous": True, "role": "Reviewer", "type": "author"}]
        contrib_type = "author"
        contributors = parse.build_contributors(authors, contrib_type)
        self.assertEqual(contributors[0].anonymous, True)
        self.assertEqual(contributors[0].surname, None)
        self.assertEqual(contributors[0].roles, [])


class TestBuildPreprint(unittest.TestCase):
    def test_build_preprint_no_events(self):
        events = []
        self.assertIsNone(parse.build_preprint(events))

    def test_build_preprint_no_event_type(self):
        events = [OrderedDict([("uri", "https://not-a-print.example.org")])]
        self.assertIsNone(parse.build_preprint(events))

    def test_build_preprint_not_preprint(self):
        events = [
            OrderedDict(
                [
                    ("event_type", "not_a_preprint"),
                    ("uri", "https://not-a-print.example.org"),
                ]
            )
        ]
        self.assertIsNone(parse.build_preprint(events))

    def test_build_preprint_uri(self):
        uri = "https://example.org/preprint/"
        events = [OrderedDict([("event_type", "preprint"), ("uri", uri)])]
        expected = Preprint(uri=uri)
        self.assertEqual(str(parse.build_preprint(events)), str(expected))

    def test_build_preprint_doi(self):
        "use an eLife article DOI as the preprint doi for testing purposes"
        doi = "10.7554/eLife.00666"
        uri = "https://doi.org/%s" % doi
        events = [OrderedDict([("event_type", "preprint"), ("uri", uri)])]
        expected = Preprint(uri=uri, doi=doi)
        self.assertEqual(str(parse.build_preprint(events)), str(expected))

    def test_build_preprint_two_preprints(self):
        uri_one = "https://example.org/preprint_one/"
        uri_two = "https://example.org/preprint_one/"
        events = [
            OrderedDict([("event_type", "preprint"), ("uri", uri_one)]),
            OrderedDict([("event_type", "preprint"), ("uri", uri_two)]),
        ]
        expected = Preprint(uri=uri_one)
        self.assertEqual(str(parse.build_preprint(events)), str(expected))



import unittest
from elifetools import utils as etoolsutils
from elifearticle import parse
from tests import XLS_PATH


class TestParseDeep(unittest.TestCase):
    def test_parse_article_02935_simple(self):
        "some simple comparisons and count list items"
        article_object, error_count = parse.build_article_from_xml(
            XLS_PATH + "elife-02935-v2.xml", detail="full"
        )
        # test pretty method for test coverage
        self.assertIsNotNone(article_object.pretty())
        # list of individual comparisons of interest
        self.assertEqual(article_object.doi, "10.7554/eLife.02935")
        self.assertEqual(article_object.version_doi, None)
        self.assertEqual(article_object.journal_issn, "2050-084X")
        # count contributors
        self.assertEqual(len(article_object.contributors), 180)
        self.assertEqual(
            len([c for c in article_object.contributors if c.contrib_type == "author"]),
            53,
        )
        self.assertEqual(
            len(
                [
                    c
                    for c in article_object.contributors
                    if c.contrib_type == "author non-byline"
                ]
            ),
            127,
        )
        self.assertEqual(
            len([c for c in article_object.contributors if c.collab is not None]), 3
        )
        # first contributor has no suffix
        self.assertEqual(article_object.contributors[0].suffix, None)
        # first contributor did not contribute equally
        self.assertEqual(article_object.contributors[0].equal_contrib, False)
        # first contributor affiliation
        self.assertEqual(
            article_object.contributors[0].affiliations[0].text,
            "Cancer Genome Project, Wellcome Trust Sanger Institute, Hinxton, United Kingdom",
        )
        # anonymous value
        self.assertEqual(article_object.contributors[0].anonymous, None)
        # count editors
        self.assertEqual(len(article_object.editors), 1)
        # first editor
        self.assertEqual(article_object.editors[0].contrib_type, "editor")
        self.assertEqual(article_object.editors[0].surname, "Golub")
        self.assertEqual(article_object.editors[0].given_name, "Todd")
        self.assertEqual(
            article_object.editors[0].affiliations[0].text,
            "Broad Institute, United States",
        )
        # ethics - not parsed yet
        self.assertEqual(len(article_object.ethics), 0)
        # compare dates
        self.assertEqual(
            article_object.dates.get("received").date,
            etoolsutils.date_struct(2014, 3, 28),
        )
        self.assertEqual(
            article_object.dates.get("accepted").date,
            etoolsutils.date_struct(2014, 9, 26),
        )
        self.assertEqual(
            article_object.dates.get("pub").date, etoolsutils.date_struct(2014, 10, 1)
        )
        self.assertEqual(article_object.dates.get("pub").day, "01")
        self.assertEqual(article_object.dates.get("pub").month, "10")
        self.assertEqual(article_object.dates.get("pub").year, "2014")
        self.assertEqual(
            article_object.dates.get("pub").publication_format, "electronic"
        )
        # datasets
        self.assertEqual(len(article_object.datasets), 2)
        self.assertEqual(article_object.datasets[0].accession_id, "EGAS00001000968")
        # review_articles
        self.assertEqual(len(article_object.review_articles), 2)
        # review_article 1
        self.assertEqual(
            article_object.review_articles[0].doi, "10.7554/eLife.02935.027"
        )
        self.assertEqual(
            article_object.review_articles[0].article_type, "article-commentary"
        )
        self.assertEqual(article_object.review_articles[0].id, "SA1")
        self.assertEqual(article_object.review_articles[0].title, "Decision letter")
        self.assertEqual(
            article_object.review_articles[0].license.href,
            "http://creativecommons.org/publicdomain/zero/1.0/",
        )
        self.assertEqual(len(article_object.review_articles[0].contributors), 1)
        self.assertEqual(
            article_object.review_articles[0].contributors[0].surname, "Golub"
        )
        self.assertEqual(
            article_object.review_articles[0].contributors[0].given_name, "Todd"
        )
        self.assertEqual(
            article_object.review_articles[0].contributors[0].contrib_type, "editor"
        )
        self.assertEqual(
            article_object.review_articles[0].contributors[0].affiliations[0].text,
            "Broad Institute, United States",
        )
        self.assertEqual(
            article_object.review_articles[0].related_articles[0].doi,
            "10.7554/eLife.02935",
        )
        self.assertEqual(
            article_object.review_articles[0].related_articles[0].article_type,
            "research-article",
        )
        # review_articles 2
        self.assertEqual(
            article_object.review_articles[1].doi, "10.7554/eLife.02935.028"
        )
        self.assertEqual(article_object.review_articles[1].article_type, "reply")
        self.assertEqual(article_object.review_articles[1].id, "SA2")
        self.assertEqual(article_object.review_articles[1].title, "Author response")
        # related_articles
        self.assertEqual(len(article_object.related_articles), 0)
        # funding
        self.assertEqual(len(article_object.funding_awards), 16)
        # award
        self.assertEqual(
            article_object.funding_awards[5].awards[0].award_id, "ALTF 1203_2012"
        )
        # keywords
        self.assertEqual(len(article_object.author_keywords), 6)
        # categories
        self.assertEqual(len(article_object.article_categories), 1)
        # display channel
        self.assertEqual(article_object.display_channel, "Research Article")
        # research organism
        self.assertEqual(len(article_object.research_organisms), 1)
        # components
        self.assertEqual(len(article_object.component_list), 28)
        # refs
        self.assertEqual(len(article_object.ref_list), 59)
        # self_uri_list
        self.assertEqual(len(article_object.self_uri_list), 1)
        self.assertIsNotNone(article_object.get_self_uri("pdf"))
        self.assertEqual(
            article_object.get_self_uri("pdf").xlink_href, "elife-02935-v2.pdf"
        )
        # version
        self.assertEqual(article_object.version, 2)
        # publisher_name
        self.assertEqual(
            article_object.publisher_name, "eLife Sciences Publications, Ltd"
        )
        # issue
        self.assertEqual(article_object.issue, None)
        # license
        self.assertEqual(
            article_object.license.href,
            "http://creativecommons.org/publicdomain/zero/1.0/",
        )
        self.assertEqual(article_object.license.copyright_statement, None)
        # elocation_id
        self.assertEqual(article_object.elocation_id, "e02935")
        # manuscript
        self.assertEqual(article_object.manuscript, "02935")
        # publisher_id pii
        self.assertEqual(article_object.pii, "02935")

    def test_parse_article_00666_simple(self):
        "some simple comparisons and count list items"
        article_object, error_count = parse.build_article_from_xml(
            XLS_PATH + "elife-00666.xml", detail="full"
        )
        # list of individual comparisons of interest
        self.assertEqual(article_object.doi, "10.7554/eLife.00666")
        self.assertEqual(article_object.journal_issn, "2050-084X")
        # count contributors
        self.assertEqual(len(article_object.contributors), 14)
        self.assertEqual(
            len([c for c in article_object.contributors if c.contrib_type == "author"]),
            4,
        )
        self.assertEqual(
            len(
                [
                    c
                    for c in article_object.contributors
                    if c.contrib_type == "on-behalf-of"
                ]
            ),
            1,
        )
        self.assertEqual(
            len(
                [
                    c
                    for c in article_object.contributors
                    if c.contrib_type == "author non-byline"
                ]
            ),
            9,
        )
        self.assertEqual(
            len([c for c in article_object.contributors if c.collab is not None]), 3
        )
        # first contributor has a suffix
        self.assertEqual(article_object.contributors[0].suffix, "Jnr")
        # first contributor contributed equally
        self.assertEqual(article_object.contributors[0].equal_contrib, True)
        # first contributor has one conflict of interest
        self.assertEqual(len(article_object.contributors[0].conflict), 1)
        self.assertEqual(article_object.contributors[0].conflict, ["Chair of JATS4R"])
        # first contributor affiliation
        self.assertEqual(
            article_object.contributors[0].affiliations[0].department,
            "Department of Production",
        )
        self.assertEqual(
            article_object.contributors[0].affiliations[0].institution,
            "eLife",
        )
        self.assertEqual(
            article_object.contributors[0].affiliations[0].city,
            "Cambridge",
        )
        self.assertEqual(
            article_object.contributors[0].affiliations[0].country,
            "United Kingdom",
        )
        self.assertEqual(
            article_object.contributors[0].affiliations[0].ror,
            None,
        )
        self.assertEqual(
            article_object.contributors[0].affiliations[0].text,
            "Department of Production, eLife, Cambridge, United Kingdom",
        )
        # ethics - not parsed yet
        self.assertEqual(len(article_object.ethics), 0)
        # compare dates
        self.assertEqual(
            article_object.dates.get("received").date,
            etoolsutils.date_struct(2016, 3, 1),
        )
        self.assertEqual(
            article_object.dates.get("accepted").date,
            etoolsutils.date_struct(2016, 4, 1),
        )
        self.assertEqual(
            article_object.dates.get("publication").date,
            etoolsutils.date_struct(2016, 4, 25),
        )
        self.assertEqual(article_object.dates.get("publication").day, "25")
        self.assertEqual(article_object.dates.get("publication").month, "04")
        self.assertEqual(article_object.dates.get("publication").year, "2016")
        self.assertEqual(
            article_object.dates.get("publication").publication_format, "electronic"
        )
        # datasets
        self.assertEqual(len(article_object.datasets), 3)
        self.assertEqual(len(article_object.get_datasets("datasets")), 1)
        self.assertEqual(len(article_object.get_datasets("prev_published_datasets")), 2)
        self.assertEqual(len(article_object.datasets[0].authors), 2)
        self.assertEqual(article_object.datasets[0].dataset_type, "datasets")
        self.assertEqual(article_object.datasets[0].year, "2016")
        self.assertEqual(article_object.datasets[0].title, "xml-mapping")
        self.assertEqual(
            article_object.datasets[0].comment, "Publicly available on GitHub"
        )
        self.assertEqual(
            article_object.datasets[0].uri,
            "https://github.com/elifesciences/XML-mapping/blob/master/elife-00666.xml",
        )
        self.assertEqual(article_object.datasets[0].assigning_authority, None)
        self.assertEqual(article_object.datasets[2].doi, "10.5061/dryad.cv323")
        self.assertEqual(
            article_object.datasets[2].assigning_authority, "Dryad Digital Repository"
        )
        # review_articles
        self.assertEqual(len(article_object.review_articles), 2)
        self.assertEqual(
            article_object.review_articles[0].doi, "10.7554/eLife.00666.029"
        )
        self.assertEqual(
            article_object.review_articles[1].doi, "10.7554/eLife.00666.030"
        )
        # review_article 1
        self.assertEqual(len(article_object.review_articles[0].contributors), 3)
        self.assertEqual(
            article_object.review_articles[0].contributors[0].surname, "Collings"
        )
        self.assertEqual(
            article_object.review_articles[0].contributors[0].contrib_type, "editor"
        )
        self.assertEqual(
            article_object.review_articles[0].contributors[1].surname, "Darian-Smith"
        )
        self.assertEqual(
            article_object.review_articles[0].contributors[1].contrib_type, "reviewer"
        )
        self.assertEqual(
            article_object.review_articles[0].contributors[2].surname, "Smith"
        )
        self.assertEqual(
            article_object.review_articles[0].contributors[2].contrib_type, "reviewer"
        )
        # related_articles
        self.assertEqual(len(article_object.related_articles), 0)
        # funding
        self.assertEqual(len(article_object.funding_awards), 2)
        # keywords
        self.assertEqual(len(article_object.author_keywords), 4)
        # categories
        self.assertEqual(len(article_object.article_categories), 2)
        # display channel
        self.assertEqual(article_object.display_channel, "Research Article")
        # research organism
        self.assertEqual(len(article_object.research_organisms), 2)
        # components
        self.assertEqual(len(article_object.component_list), 39)
        # component id examples
        self.assertEqual(article_object.component_list[0].id, None)
        self.assertEqual(article_object.component_list[2].id, "table1")
        # component type examples
        self.assertEqual(article_object.component_list[0].type, "abstract")
        self.assertEqual(article_object.component_list[2].type, "table-wrap")
        self.assertEqual(
            article_object.component_list[5].type, "supplementary-material"
        )
        self.assertEqual(article_object.component_list[9].type, "fig")
        self.assertEqual(article_object.component_list[13].type, "media")
        self.assertEqual(
            article_object.component_list[15].type, "supplementary-material"
        )
        self.assertEqual(article_object.component_list[19].type, "boxed-text")
        self.assertEqual(
            article_object.component_list[22].type, "supplementary-material"
        )
        self.assertEqual(article_object.component_list[34].type, "sub-article")
        self.assertEqual(article_object.component_list[35].type, "sub-article")
        # component asset examples
        self.assertEqual(article_object.component_list[0].asset, None)
        self.assertEqual(article_object.component_list[2].asset, None)
        self.assertEqual(article_object.component_list[5].asset, "data")
        self.assertEqual(article_object.component_list[9].asset, "figsupp")
        self.assertEqual(article_object.component_list[13].asset, "media")
        self.assertEqual(article_object.component_list[15].asset, "code")
        self.assertEqual(article_object.component_list[19].asset, None)
        self.assertEqual(article_object.component_list[22].asset, "supp")
        self.assertEqual(article_object.component_list[34].asset, "dec")
        self.assertEqual(article_object.component_list[35].asset, "resp")
        # refs
        self.assertEqual(len(article_object.ref_list), 54)
        # license
        self.assertEqual(
            article_object.license.href, "http://creativecommons.org/licenses/by/4.0/"
        )
        self.assertEqual(
            article_object.license.copyright_statement, "\u00a9 2016, Harrison et al"
        )
        # elocation_id
        self.assertEqual(article_object.elocation_id, "e00666")
        # manuscript
        self.assertEqual(article_object.manuscript, "00666")
        # publisher_id pii
        self.assertEqual(article_object.pii, "00666")

    def test_parse_article_cstp77_simple(self):
        "some simple comparisons and count list items"
        article_object, error_count = parse.build_article_from_xml(
            XLS_PATH + "cstp77-jats.xml"
        )
        # list of individual comparisons of interest
        self.assertEqual(article_object.doi, "10.5334/cstp.77")
        self.assertEqual(article_object.journal_issn, "2057-4991")
        self.assertEqual(
            article_object.journal_title, "Citizen Science: Theory and Practice"
        )
        # count contributors
        self.assertEqual(len(article_object.contributors), 4)
        self.assertEqual(
            len([c for c in article_object.contributors if c.contrib_type == "author"]),
            4,
        )

        # compare dates
        self.assertEqual(
            article_object.dates.get("received").date,
            etoolsutils.date_struct(2016, 8, 11),
        )
        self.assertEqual(
            article_object.dates.get("accepted").date,
            etoolsutils.date_struct(2017, 3, 28),
        )
        self.assertEqual(
            article_object.dates.get("pub").date, etoolsutils.date_struct(2017, 7, 4)
        )
        self.assertEqual(article_object.dates.get("pub").day, "04")
        self.assertEqual(article_object.dates.get("pub").month, "07")
        self.assertEqual(article_object.dates.get("pub").year, "2017")
        self.assertEqual(
            article_object.dates.get("pub").publication_format, "electronic"
        )
        # keywords
        self.assertEqual(len(article_object.author_keywords), 4)
        # refs
        self.assertEqual(len(article_object.ref_list), 36)
        # publisher_name
        self.assertEqual(article_object.publisher_name, "Ubiquity Press")
        # issue
        self.assertEqual(article_object.issue, "1")
        # elocation_id
        self.assertEqual(article_object.elocation_id, "3")
        # manuscript
        self.assertEqual(article_object.manuscript, "77")
        # publisher_id pii
        self.assertEqual(article_object.pii, None)
        # review_articles, should be empty
        self.assertEqual(len(article_object.review_articles), 0)

    def test_parse_article_1234567890_simple(self):
        "some simple comparisons and count list items"
        article_object, error_count = parse.build_article_from_xml(
            XLS_PATH + "elife-1234567890-v2.xml", detail="full"
        )
        # list of individual comparisons of interest
        self.assertEqual(article_object.doi, "10.7554/eLife.1234567890")
        self.assertEqual(article_object.journal_issn, "2050-084X")
        self.assertEqual(article_object.journal_title, "eLife")
        # count contributors
        self.assertEqual(len(article_object.contributors), 7)
        self.assertEqual(
            len([c for c in article_object.contributors if c.contrib_type == "author"]),
            6,
        )

        # compare dates
        self.assertEqual(
            article_object.dates.get("received"),
            None,
        )
        self.assertEqual(
            article_object.dates.get("accepted"),
            None,
        )
        self.assertEqual(
            article_object.dates.get("publication").date,
            etoolsutils.date_struct(2023, 10, 22),
        )
        self.assertEqual(article_object.dates.get("publication").day, "22")
        self.assertEqual(article_object.dates.get("publication").month, "10")
        self.assertEqual(article_object.dates.get("publication").year, "2023")
        self.assertEqual(
            article_object.dates.get("publication").publication_format, "electronic"
        )
        # keywords
        self.assertEqual(len(article_object.author_keywords), 6)
        # refs
        self.assertEqual(len(article_object.ref_list), 18)
        # publisher_name
        self.assertEqual(
            article_object.publisher_name, "eLife Sciences Publications, Ltd"
        )
        # issue
        self.assertEqual(article_object.issue, None)
        # elocation_id
        self.assertEqual(article_object.elocation_id, "RP1234567890")
        # manuscript
        self.assertEqual(article_object.manuscript, "1234567890")
        # publisher_id pii
        self.assertEqual(article_object.pii, "1234567890")
        # review_articles
        self.assertEqual(len(article_object.review_articles), 8)
        # preprint
        self.assertIsNotNone(article_object.preprint)
        self.assertEqual(
            article_object.preprint.uri, "https://doi.org/10.1101/2021.11.09.467796"
        )
        self.assertEqual(article_object.preprint.doi, "10.1101/2021.11.09.467796")
        # publication history events
        self.assertEqual(len(article_object.publication_history), 4)
        self.assertEqual(
            article_object.publication_history[0].uri,
            "https://doi.org/10.1101/2021.11.09.467796",
        )
        self.assertEqual(
            article_object.publication_history[0].doi, "10.1101/2021.11.09.467796"
        )
        self.assertEqual(
            article_object.publication_history[1].uri,
            "https://doi.org/10.7554/eLife.1234567890.1",
        )
        self.assertEqual(
            article_object.publication_history[1].doi, "10.7554/eLife.1234567890.1"
        )
        self.assertEqual(
            article_object.publication_history[1].event_type, "reviewed-preprint"
        )
        self.assertEqual(
            article_object.publication_history[1].event_desc,
            "This manuscript was published as a reviewed preprint.",
        )
        self.assertEqual(
            article_object.publication_history[1].date,
            etoolsutils.date_struct(2023, 4, 15),
        )



from setuptools import setup

import elifearticle

with open("README.md") as fp:
    readme = fp.read()

setup(
    name="elifearticle",
    version=elifearticle.__version__,
    description="eLife article objects.",
    long_description=readme,
    long_description_content_type="text/markdown",
    packages=["elifearticle"],
    license="MIT",
    install_requires=["elifetools>=0.33.0", "GitPython"],
    url="https://github.com/elifesciences/elife-article",
    maintainer="eLife Sciences Publications Ltd.",
    maintainer_email="tech-team@elifesciences.org",
    classifiers=[
        "Development Status :: 3 - Alpha",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python :: 3",
    ],
)



import unittest
from collections import OrderedDict
from elifearticle import parse


class TestParseBuildReviewArticles(unittest.TestCase):
    def setUp(self):
        pass

    def test_build_review_articles(self):
        "test building an editor's evaluation review article"
        sub_articles_data = [
            OrderedDict(
                [
                    ("doi", "10.7554/eLife.00666.sa0"),
                    ("article_type", "editor-report"),
                    ("id", "sa0"),
                    ("article_title", "Editor's evaluation"),
                    (
                        "contributors",
                        [
                            {
                                "type": "author",
                                "role": "Reviewing Editor",
                                "surname": "Ma",
                                "given-names": "Yuting",
                                "affiliations": [
                                    {
                                        "institution": "Suzhou Institute of Systems Medicine",
                                        "country": "China",
                                    }
                                ],
                            }
                        ],
                    ),
                    (
                        "related_objects",
                        [
                            OrderedDict(
                                [
                                    ("id", "ro1"),
                                    ("link_type", "continued-by"),
                                    (
                                        "xlink_href",
                                        "https://sciety.org/articles/activity/10.1101/2020.11.21.391326",
                                    ),
                                ]
                            )
                        ],
                    ),
                    ("parent_doi", "10.7554/eLife.00666"),
                    ("parent_article_type", "research-article"),
                    ("parent_article_title", "The eLife research article"),
                    (
                        "parent_license_url",
                        "http://creativecommons.org/licenses/by/4.0/",
                    ),
                ]
            )
        ]
        review_articles = parse.build_review_articles(sub_articles_data)
        self.assertEqual(review_articles[0].doi, "10.7554/eLife.00666.sa0")
        self.assertEqual(review_articles[0].title, "Editor's evaluation")
        self.assertEqual(review_articles[0].article_type, "editor-report")
        self.assertEqual(review_articles[0].id, "sa0")
        # contributor
        self.assertEqual(review_articles[0].contributors[0].contrib_type, "author")
        self.assertEqual(review_articles[0].contributors[0].surname, "Ma")
        self.assertEqual(review_articles[0].contributors[0].given_name, "Yuting")
        self.assertEqual(
            review_articles[0].contributors[0].affiliations[0].text,
            "Suzhou Institute of Systems Medicine, China",
        )
        # license
        self.assertEqual(
            review_articles[0].license.href,
            "http://creativecommons.org/licenses/by/4.0/",
        )
        # related objects
        self.assertEqual(review_articles[0].related_objects[0].id, "ro1")
        self.assertEqual(
            review_articles[0].related_objects[0].link_type, "continued-by"
        )
        self.assertEqual(
            review_articles[0].related_objects[0].xlink_href,
            "https://sciety.org/articles/activity/10.1101/2020.11.21.391326",
        )
        # related article
        self.assertEqual(
            review_articles[0].related_articles[0].article_type, "research-article"
        )
        self.assertEqual(
            review_articles[0].related_articles[0].doi, "10.7554/eLife.00666"
        )
